<?php

namespace App\Http\Controllers;

use App\Models\Supplier;
use App\Models\Utility;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

class SupplierController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth'); // Assurez-vous que l'utilisateur est authentifié avant de pouvoir accéder à cette méthode
    }

    // Index - Affichage des fournisseurs
    public function index(Request $request)
    {
        // Vérifier si l'utilisateur a la permission de gérer les fournisseurs
        if (Auth::user()->can('manage supplier')) {
            // Récupérer les catégories de fournisseurs (par exemple : matériel, service)
            $categories = ['material' => 'Materiel', 'service' => 'Service'];
            
            // Ajouter une option de sélection par défaut
            $categories = collect($categories)->prepend('Select Category', '');
            
            // Si une catégorie est sélectionnée dans la requête, filtrer les fournisseurs par catégorie
            if (!empty($request->category)) {
                $suppliers = Supplier::where('category', $request->category)->get();
            } else {
                // Si aucune catégorie n'est sélectionnée, afficher tous les fournisseurs
                $suppliers = Supplier::all();
            }

            // Retourner la vue avec les fournisseurs et les catégories
            return view('supplier.index', compact('suppliers', 'categories'));
        } else {
            // Si l'utilisateur n'a pas la permission, le rediriger avec un message d'erreur
            return redirect()->back()->with('error', __('Permission denied.'));
        }
    }

    // Formulaire de création
    public function create()
    {
        if (Auth::user()->can('manage supplier')) {
            return view('supplier.create');
        } else {
            return response()->json(['error' => __('Permission denied.')], 401);
        }
    }

    // Enregistrer un nouveau fournisseur
    public function store(Request $request)
{
    // Vérification des permissions
    if (Auth::user()->can('manage supplier')) {

        // Validation des champs
        $rules = [
            'name' => 'required|string|max:255',
            'email' => [
                'email',
                Rule::unique('supplier') 
            ],
            'phone' => 'required|string|max:10',
            'address' => 'required|string|max:255',
            'category' => 'required|in:service,materiel',
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            $messages = $validator->getMessageBag();
            return redirect()->route('supplier.index')->with('error', $messages->first());
        }

        // Créer un nouvel enregistrement Supplier
        $supplier = new Supplier();
        $supplier->name = $request->name;
        $supplier->email = $request->email;
        $supplier->phone = $request->phone;
        $supplier->address = $request->address;
        $supplier->category = $request->category;

        // Sauvegarder le fournisseur dans la base de données
        $supplier->save();

        // Redirection avec message de succès
        return redirect()->route('supplier.index')->with('success', __('Supplier successfully created.'));
    } else {
        return redirect()->back()->with('error', __('Permission denied.'));
    }
}

public function show($id)
{
    if (Auth::user()->can('manage supplier')) {
        $supplier = Supplier::findOrFail($id);
        return view('supplier.show', compact('supplier')); // Renvoyer la vue des détails
    } else {
        return redirect()->back()->with('error', __('Permission denied.'));
    }
}



public function edit($id)
{
    $supplier = Supplier::findOrFail($id);
    return view('supplier.edit', compact('supplier'));
}

// app/Http/Controllers/SupplierController.php

public function update(Request $request, $id)
{
    // Validation des données
    $request->validate([
        'name' => 'required|string|max:255',
        'phone' => 'required|string|max:10',
        'email' => 'required|email|max:255',
        'address' => 'required|string|max:255',
        'category' => 'required|string|max:255',
    ]);

    // Trouver le fournisseur par son ID
    $supplier = Supplier::findOrFail($id);

    // Mettre à jour les données du fournisseur
    $supplier->update([
        'name' => $request->input('name'),
        'phone' => $request->input('phone'),
        'email' => $request->input('email'),
        'address' => $request->input('address'),
        'category' => $request->input('category'),
    ]);

    // Retourner une réponse ou rediriger
    return redirect()->route('supplier.index')->with('success', 'Supplier updated successfully.');
}

public function destroy($id)
{
    $supplier = Supplier::findOrFail($id);
    $supplier->delete();

    return redirect()->route('supplier.index')->with('success', 'Supplier deleted successfully');
}





}
